@@ -110,7 +110,7 @@ static GConfEnumStringPair panel_type_type_enum_map [] = {
 static GConfEnumStringPair background_type_enum_map [] = {
 	{ PANEL_BACK_NONE,   "no-background" },
 	{ PANEL_BACK_COLOR,  "color-background" },
-	{ PANEL_BACK_PIXMAP, "pixmap-background" },
+	{ PANEL_BACK_IMAGE,  "pixmap-background" },
 };
 
 static GConfEnumStringPair panel_size_type_enum_map [] = {
@@ -281,7 +281,7 @@ back_change (AppletInfo  *info,
 {
 	if (info->type == APPLET_BONOBO)
 		panel_applet_frame_change_background (
-			PANEL_APPLET_FRAME (info->widget), panel->back_type);
+			PANEL_APPLET_FRAME (info->widget), panel->background.type);
 }
 
 static void
@@ -672,7 +672,7 @@ panel_do_popup_menu (PanelWidget *panel,
 		menu = make_popup_panel_menu (panel);
 		if (basep) {
                         basep->autohide_inhibit = TRUE;
-			basep_widget_autohide (basep);
+			basep_widget_queue_autohide (basep);
 		}
 
 		gtk_menu_set_screen (GTK_MENU (menu),
@@ -1085,11 +1085,14 @@ drop_background_reset (PanelWidget *panel)
 				    panel->orient,
 				    panel->sz,
 				    PANEL_BACK_NONE,
-				    panel->back_pixmap,
-				    panel->fit_pixmap_bg,
-				    panel->stretch_pixmap_bg,
-				    panel->rotate_pixmap_bg,
-				    &panel->back_color);
+				    panel->background.image,
+				    panel->background.fit_image,
+				    panel->background.stretch_image,
+				    panel->background.rotate_image,
+				    &panel->background.color);
+
+	if (BASEP_IS_WIDGET (panel->panel_parent))
+		basep_update_frame (BASEP_WIDGET (panel->panel_parent));
 }
 
 static void
@@ -1103,6 +1106,9 @@ drop_bgimage (PanelWidget *panel, const char *bgimage)
 
 		g_free (filename);
 	}
+
+	if (BASEP_IS_WIDGET (panel->panel_parent))
+		basep_update_frame (BASEP_WIDGET (panel->panel_parent));
 }
 
 static void
@@ -1126,8 +1132,14 @@ drop_internal_icon (PanelWidget *panel, int pos, const char *icon_name,
 		launcher_hoard (launcher);
 
 		if (old_launcher != NULL &&
-		    old_launcher->button != NULL)
-			gtk_widget_destroy (old_launcher->button);
+		    old_launcher->button != NULL) {
+			if (old_launcher->prop_dialog != NULL) {
+				g_signal_handler_disconnect ( old_launcher->button,
+						old_launcher->destroy_handler);
+				launcher_properties_destroy(old_launcher);
+			}
+			panel_applet_remove_in_idle (old_launcher->info);
+		}
 	}
 }
 
@@ -1202,17 +1214,20 @@ drop_color (PanelWidget *panel,
 	    int          pos,
 	    guint16     *dropped)
 {
-	PanelColor c;
+	PanelColor color;
 
 	if (!dropped)
 		return;
 
-	c.red   = dropped [0];
-	c.green = dropped [1];
-	c.blue  = dropped [2];
-	c.alpha = 65535;
+	color.gdk.red   = dropped [0];
+	color.gdk.green = dropped [1];
+	color.gdk.blue  = dropped [2];
+	color.alpha     = 65535;
 
-	panel_widget_set_back_color (panel, &c);
+	panel_widget_set_back_color (panel, &color);
+
+	if (BASEP_IS_WIDGET (panel->panel_parent))
+		basep_update_frame (BASEP_WIDGET (panel->panel_parent));
 }
 
 static GtkTargetList *
@@ -1242,28 +1257,24 @@ get_target_list (void)
 	return target_list;
 }
 
-static gboolean
-is_this_drop_ok (GtkWidget      *widget,
-		 GdkDragContext *context,
-		 guint          *ret_info,
-		 GdkAtom        *ret_atom)
+gboolean
+panel_check_dnd_target_data (GtkWidget      *widget,
+			     GdkDragContext *context,
+			     guint          *ret_info,
+			     GdkAtom        *ret_atom)
 {
-	GtkWidget *panel;
-	GList     *l;
+	GList *l;
 
 	g_return_val_if_fail (widget, FALSE);
 
-	if (!BASEP_IS_WIDGET (widget) && !FOOBAR_IS_WIDGET (widget))
+	if (!BASEP_IS_WIDGET  (widget) &&
+	    !FOOBAR_IS_WIDGET (widget) &&
+	    !BUTTON_IS_WIDGET (widget))
 		return FALSE;
 
 	if (!(context->actions & (GDK_ACTION_COPY|GDK_ACTION_MOVE)))
 		return FALSE;
 
-	if (BASEP_IS_WIDGET (widget))
-		panel = BASEP_WIDGET (widget)->panel;
-	else
-		panel = FOOBAR_WIDGET (widget)->panel;
-
 	for (l = context->targets; l; l = l->next) {
 		GdkAtom atom;
 		guint   info;
@@ -1309,6 +1320,46 @@ do_highlight (GtkWidget *widget, gboolean highlight)
 	}
 }
 
+gboolean
+panel_check_drop_forbidden (PanelWidget    *panel,
+			    GdkDragContext *context,
+			    guint           info,
+			    guint           time_)
+{
+	if (!panel)
+		return FALSE;
+
+	if (info == TARGET_APPLET_INTERNAL) {
+		GtkWidget *source_widget;
+
+		source_widget = gtk_drag_get_source_widget (context);
+
+		if (BUTTON_IS_WIDGET (source_widget)) {
+			GSList *forb;
+
+			forb = g_object_get_data (G_OBJECT (source_widget),
+						  PANEL_APPLET_FORBIDDEN_PANELS);
+
+			if (g_slist_find (forb, panel))
+				return FALSE;
+		}
+	}
+
+	if (info == TARGET_ICON_INTERNAL ||
+	    info == TARGET_APPLET_INTERNAL) {
+		if (context->actions & GDK_ACTION_MOVE)
+			gdk_drag_status (context, GDK_ACTION_MOVE, time_);
+		else
+			gdk_drag_status (context, context->suggested_action, time_);
+
+	} else if (context->actions & GDK_ACTION_COPY)
+		gdk_drag_status (context, GDK_ACTION_COPY, time_);
+	else
+		gdk_drag_status (context, context->suggested_action, time_);
+
+	return TRUE;
+
+}
 
 static gboolean
 drag_motion_cb (GtkWidget	   *widget,
@@ -1317,50 +1368,22 @@ drag_motion_cb (GtkWidget	   *widget,
 		gint                y,
 		guint               time)
 {
-	guint info;
+	PanelWidget *panel = NULL;
+	guint        info;
 
-	if ( ! is_this_drop_ok (widget, context, &info, NULL))
+	if (!panel_check_dnd_target_data (widget, context, &info, NULL))
 		return FALSE;
 
-	/* check forbiddenness */
-	if (info == TARGET_APPLET_INTERNAL) {
-		GtkWidget *source_widget;
+	if (BASEP_IS_WIDGET (widget)) {
+		BasePWidget *basep = BASEP_WIDGET (widget);
 
-		source_widget = gtk_drag_get_source_widget (context);
-		if (source_widget != NULL &&
-		    BUTTON_IS_WIDGET (source_widget)) {
-			GSList *forb;
-			PanelWidget *panel = NULL;
+		panel = PANEL_WIDGET (basep->panel);
 
-			if (BASEP_IS_WIDGET (widget)) {
-				BasePWidget *basep =
-					BASEP_WIDGET (widget);
-				panel = PANEL_WIDGET (basep->panel);
-			} else if (FOOBAR_IS_WIDGET (widget)) {
-				panel = PANEL_WIDGET (FOOBAR_WIDGET (widget)->panel);
-			}
-			forb = g_object_get_data (G_OBJECT (source_widget),
-						  PANEL_APPLET_FORBIDDEN_PANELS);
-			if (panel != NULL &&
-			    g_slist_find (forb, panel) != NULL)
-				return FALSE;
-		}
-	}
+	} else if (FOOBAR_IS_WIDGET (widget))
+		panel = PANEL_WIDGET (FOOBAR_WIDGET (widget)->panel);
 
-	/* always prefer copy, except for internal icons/applets,
-	 * where we prefer move */
-	if (info == TARGET_ICON_INTERNAL ||
-	    info == TARGET_APPLET_INTERNAL) {
-		if (context->actions & GDK_ACTION_MOVE) {
-			gdk_drag_status (context, GDK_ACTION_MOVE, time);
-		} else {
-			gdk_drag_status (context, context->suggested_action, time);
-		}
-	} else if (context->actions & GDK_ACTION_COPY) {
-		gdk_drag_status (context, GDK_ACTION_COPY, time);
-	} else {
-		gdk_drag_status (context, context->suggested_action, time);
-	}
+	if (!panel_check_drop_forbidden (panel, context, info, time))
+		return FALSE;
 
 	do_highlight (widget, TRUE);
 
@@ -1382,7 +1405,7 @@ drag_drop_cb (GtkWidget	        *widget,
 {
 	GdkAtom ret_atom = 0;
 
-	if ( ! is_this_drop_ok (widget, context, NULL, &ret_atom))
+	if (!panel_check_dnd_target_data (widget, context, NULL, &ret_atom))
 		return FALSE;
 
 	gtk_drag_get_data(widget, context,
@@ -1400,6 +1423,60 @@ drag_leave_cb (GtkWidget	*widget,
 	do_highlight (widget, FALSE);
 }
 
+void
+panel_receive_dnd_data (PanelWidget      *panel,
+			guint             info,
+			int               pos,
+			GtkSelectionData *selection_data,
+			GdkDragContext   *context,
+			guint             time_,
+			gboolean          is_foobar)
+{
+	switch (info) {
+	case TARGET_URL:
+		drop_urilist (panel, pos, (char *)selection_data->data, !is_foobar);
+		break;
+	case TARGET_NETSCAPE_URL:
+		drop_url (panel, pos, (char *)selection_data->data);
+		break;
+	case TARGET_COLOR:
+		drop_color (panel, pos, (guint16 *)selection_data->data);
+		break;
+	case TARGET_BGIMAGE:
+		if (is_foobar)
+			drop_bgimage (panel, (char *)selection_data->data);
+		break;
+	case TARGET_BACKGROUND_RESET:
+		if (is_foobar)
+			drop_background_reset (panel);
+		break;
+	case TARGET_DIRECTORY:
+		drop_directory (panel, pos, (char *)selection_data->data);
+		break;
+	case TARGET_APPLET:
+		if (!selection_data->data) {
+			gtk_drag_finish (context, FALSE, FALSE, time_);
+			return;
+		}
+		panel_applet_frame_load ((char *)selection_data->data,
+					 panel, pos, TRUE, NULL);
+		break;
+	case TARGET_APPLET_INTERNAL:
+		drop_internal_applet (panel, pos, (char *)selection_data->data,
+				      context->action);
+		break;
+	case TARGET_ICON_INTERNAL:
+		drop_internal_icon (panel, pos, (char *)selection_data->data,
+				    context->action);
+		break;
+	default:
+		gtk_drag_finish (context, FALSE, FALSE, time_);
+		return;
+	}
+
+	gtk_drag_finish (context, TRUE, FALSE, time_);
+}
+
 static void
 drag_data_recieved_cb (GtkWidget	*widget,
 		       GdkDragContext   *context,
@@ -1419,7 +1496,7 @@ drag_data_recieved_cb (GtkWidget	*widget,
 	/* we use this only to really find out the info, we already
 	   know this is an ok drop site and the info that got passed
 	   to us is bogus (it's always 0 in fact) */
-	if ( ! is_this_drop_ok (widget, context, &info, NULL)) {
+	if (!panel_check_dnd_target_data (widget, context, &info, NULL)) {
 		gtk_drag_finish (context, FALSE, FALSE, time);
 		return;
 	}
@@ -1441,50 +1518,9 @@ drag_data_recieved_cb (GtkWidget	*widget,
 	else if(pos > panel->size)
 		pos = panel->size;
 
-	switch (info) {
-	case TARGET_URL:
-		drop_urilist (panel, pos, (char *)selection_data->data,
-			      FOOBAR_IS_WIDGET(widget) ? FALSE : TRUE);
-		break;
-	case TARGET_NETSCAPE_URL:
-		drop_url (panel, pos, (char *)selection_data->data);
-		break;
-	case TARGET_COLOR:
-		drop_color (panel, pos, (guint16 *)selection_data->data);
-		break;
-	case TARGET_BGIMAGE:
-		if ( ! FOOBAR_IS_WIDGET(widget))
-			drop_bgimage (panel, (char *)selection_data->data);
-		break;
-	case TARGET_BACKGROUND_RESET:
-		if ( ! FOOBAR_IS_WIDGET(widget))
-			drop_background_reset (panel);
-		break;
-	case TARGET_DIRECTORY:
-		drop_directory (panel, pos, (char *)selection_data->data);
-		break;
-	case TARGET_APPLET:
-		if ( ! selection_data->data) {
-			gtk_drag_finish (context, FALSE, FALSE, time);
-			return;
-		}
-		panel_applet_frame_load ((char *)selection_data->data,
-					 panel, pos, TRUE, NULL);
-		break;
-	case TARGET_APPLET_INTERNAL:
-		drop_internal_applet (panel, pos, (char *)selection_data->data,
-				      context->action);
-		break;
-	case TARGET_ICON_INTERNAL:
-		drop_internal_icon (panel, pos, (char *)selection_data->data,
-				    context->action);
-		break;
-	default:
-		gtk_drag_finish (context, FALSE, FALSE, time);
-		return;
-	}
-
-	gtk_drag_finish (context, TRUE, FALSE, time);
+	panel_receive_dnd_data (
+		panel, info, pos, selection_data, context,
+		time, FOOBAR_IS_WIDGET (widget));
 }
 
 static void
@@ -1969,24 +2005,24 @@ panel_load_panels_from_gconf (void)
 			panel_gconf_get_client (), key, GCONF_VALUE_STRING, NULL);
 
 	for (l = panel_ids; l; l = l->next) {
-		GtkWidget     *panel = NULL;
-		PanelType      type;
-		PanelBackType  back_type;
-		BasePState     state;
-		BasePMode      mode;
-		PanelColor     back_color = {0, 0, 0, 0xFFFF};
-		GdkColor       gdkcolor = {0, 0, 0, 1};
-		gboolean       fit_pixmap_bg;
-		gboolean       stretch_pixmap_bg;
-		gboolean       rotate_pixmap_bg;
-		gboolean       hidebuttons_enabled;
-		gboolean       hidebutton_pixmaps_enabled;
-		int            screen;
-		int            monitor;
-		int            size;
-		char          *panel_id;
-		char          *back_pixmap;
-		char          *tmp_str;
+		GtkWidget           *panel = NULL;
+		PanelType            type = EDGE_PANEL;
+		PanelBackgroundType  back_type = PANEL_BACK_NONE;
+		BasePState           state;
+		BasePMode            mode;
+		PanelColor           back_color = { { 0, 0, 0, 0 }, 0xffff};
+		GdkColor             gdkcolor = {0, 0, 0, 1};
+		gboolean             fit_pixmap_bg;
+		gboolean             stretch_pixmap_bg;
+		gboolean             rotate_pixmap_bg;
+		gboolean             hidebuttons_enabled;
+		gboolean             hidebutton_pixmaps_enabled;
+		int                  screen;
+		int                  monitor;
+		int                  size = PANEL_SIZE_SMALL;
+		char                *panel_id;
+		char                *back_pixmap;
+		char                *tmp_str;
 
 		panel_id = l->data;
 
@@ -2015,9 +2051,7 @@ panel_load_panels_from_gconf (void)
 		tmp_str = panel_get_string (profile, panel_id, "panel_background_color", NULL);
 		if (!tmp_str || !tmp_str [0]) {
 			gdk_color_parse (tmp_str, &gdkcolor);
-			back_color.red   = gdkcolor.red;
-			back_color.green = gdkcolor.green;
-			back_color.blue  = gdkcolor.blue;
+			back_color.gdk = gdkcolor;
 		}
 		g_free (tmp_str);
 
@@ -2025,7 +2059,7 @@ panel_load_panels_from_gconf (void)
 					profile, panel_id, "panel_background_color_alpha", 0xFFFF);
 
 		tmp_str = panel_get_string (
-				profile, panel_id, "panel_background_type", "no_background");
+				profile, panel_id, "panel_background_type", "no-background");
 		gconf_string_to_enum (background_type_enum_map, tmp_str, (int *) &back_type);
 		g_free (tmp_str);
 		
@@ -2052,12 +2086,12 @@ panel_load_panels_from_gconf (void)
 		mode    = panel_get_int (profile, panel_id, "panel_hide_mode", 0);
 
 		tmp_str = panel_get_string (profile, panel_id, "panel_type", "edge-panel");
-		gconf_string_to_enum (panel_type_type_enum_map, tmp_str, (gint *) &type);
+		gconf_string_to_enum (panel_type_type_enum_map, tmp_str, (int *) &type);
 		g_free (tmp_str);
 
 		switch (type) {
 		case EDGE_PANEL: {
-			BorderEdge edge;
+			BorderEdge edge = BORDER_BOTTOM;
 
 			tmp_str = panel_get_string (profile, panel_id,
 						    "screen_edge", "panel-edge-bottom");
@@ -2081,8 +2115,8 @@ panel_load_panels_from_gconf (void)
 			}
 			break;
 		case ALIGNED_PANEL: {
-			AlignedAlignment align;
-			BorderEdge       edge;
+			AlignedAlignment align = ALIGNED_LEFT;
+			BorderEdge       edge = BORDER_BOTTOM;
 
 			tmp_str = panel_get_string (profile, panel_id,
 						    "screen_edge", "panel-edge-bottom");
@@ -2114,8 +2148,8 @@ panel_load_panels_from_gconf (void)
 			}
 			break;
 		case SLIDING_PANEL: {
-			SlidingAnchor anchor;
-			BorderEdge    edge;
+			SlidingAnchor anchor = SLIDING_ANCHOR_LEFT;
+			BorderEdge    edge = BORDER_BOTTOM;
 			gint16        offset;
 
 			tmp_str = panel_get_string (profile, panel_id,
@@ -2151,7 +2185,7 @@ panel_load_panels_from_gconf (void)
 			}
 			break;
 		case DRAWER_PANEL: {
-			int orient;
+			int orient = PANEL_ORIENT_UP;
 
 			tmp_str = panel_get_string (profile, panel_id,
 						    "panel_orient", "panel-orient-up");
@@ -2176,7 +2210,7 @@ panel_load_panels_from_gconf (void)
 			}
 			break;
 		case FLOATING_PANEL: {
-			GtkOrientation orient;
+			GtkOrientation orient = GTK_ORIENTATION_HORIZONTAL;
 			int            x, y;
 
 			tmp_str = panel_get_string (profile, panel_id,
@@ -2320,24 +2354,24 @@ panel_save_to_gconf (PanelData *pd)
 
 	panel_set_bool (profile, panel->unique_id,
 			"panel_background_pixmap_fit",
-			panel->fit_pixmap_bg);
+			panel->background.fit_image);
 
 	panel_set_bool (profile, panel->unique_id,
 			"panel_background_pixmap_stretch",
-			panel->stretch_pixmap_bg);
+			panel->background.stretch_image);
 
 	panel_set_bool (profile, panel->unique_id,
 			"panel_background_pixmap_rotate",
-			panel->rotate_pixmap_bg);
+			panel->background.rotate_image);
 
 	panel_set_string (profile, panel->unique_id,
 			  "panel_background_pixmap",
-			  sure_string (panel->back_pixmap));
+			  sure_string (panel->background.image));
 
 	color = g_strdup_printf ("#%02x%02x%02x",
-			 (guint) panel->back_color.red / 256,
-			 (guint) panel->back_color.green / 256,
-			 (guint) panel->back_color.blue / 256);
+			 (guint) panel->background.color.gdk.red   / 256,
+			 (guint) panel->background.color.gdk.green / 256,
+			 (guint) panel->background.color.gdk.blue  / 256);
 	
 	panel_set_string (profile, panel->unique_id,
 			  "panel_background_color", color);
@@ -2345,11 +2379,12 @@ panel_save_to_gconf (PanelData *pd)
 
 	panel_set_int (profile, panel->unique_id,
 		       "panel_background_color_alpha",
-		       panel->back_color.alpha);
+		       panel->background.color.alpha);
 
 	panel_set_string (profile, panel->unique_id,
 			  "panel_background_type", 
-			  gconf_enum_to_string (background_type_enum_map, panel->back_type));
+			  gconf_enum_to_string (background_type_enum_map,
+			  panel->background.type));
 	
 	if (BORDER_IS_WIDGET (pd->panel))
 		panel_set_string (profile, panel->unique_id,
