@@ -6,57 +6,36 @@
  *           Miguel de Icaza
  */
 
-#include <config.h>
-#include <string.h>
-#include <signal.h>
-#include <limits.h>
-#include <unistd.h>
-#include <sys/types.h>
-#include <sys/wait.h>
+/* FIXME:
+ *
+ *  A lot of this is redundant. As far as I can
+ *  make out the only code here that still makes
+ *  sense is the setting of the restart command,
+ *  the unlinking of dead launchers and destroying
+ *  all panels on destroy.
+ */
 
-#include <gdk/gdkx.h>
-#include <X11/keysym.h>
+#include <config.h>
 
 #include <libgnome/libgnome.h>
-#include <libgnome/gnome-desktop-item.h>
-#include <libbonobo.h>
 
 #include "session.h"
 
-#include "aligned-widget.h"
-#include "button-widget.h"
-#include "distribution.h"
-#include "drawer-widget.h"
-#include "edge-widget.h"
-#include "floating-widget.h"
-#include "foobar-widget.h"
-#include "launcher.h"
-#include "logout.h"
-#include "menu-fentry.h"
-#include "menu-util.h"
+#include "applet.h"
 #include "menu.h"
-#include "panel-util.h"
-#include "panel-gconf.h"
-#include "panel-config-global.h"
-#include "session.h"
-#include "sliding-widget.h"
-#include "status.h"
-#include "swallow.h"
-#include "gnome-run.h"
-#include "xstuff.h"
-#include "multiscreen-stuff.h"
+#include "panel.h"
 #include "panel-applet-frame.h"
+#include "panel-config-global.h"
+#include "panel-gconf.h"
 #include "panel-shell.h"
+#include "xstuff.h"
 
 #undef SESSION_DEBUG
 
 extern GSList          *panels;
 extern GSList          *applets;
-extern int              applet_count;
-extern GtkTooltips     *panel_tooltips;
 extern GnomeClient     *client;
 extern GSList          *panel_list;
-extern char            *kde_menudir;
 static int              config_sync_timeout;
 
 gboolean                applets_to_sync = FALSE;
@@ -140,8 +119,6 @@ panel_session_do_save (GnomeClient *client,
 
 		panel_session_save_applets (applets);
 	}
-	/* FIXME : add hack to see if we can flush changes */
-	gconf_client_suggest_sync (panel_gconf_get_client (), NULL);
 }
 
 static guint sync_handler = 0;
@@ -252,46 +229,34 @@ int
 panel_session_die (GnomeClient *client,
 		   gpointer client_data)
 {
+	GSList *panels_to_destroy;
 	GSList *l;
 
 	g_source_remove (config_sync_timeout);
 	config_sync_timeout = 0;
   
-	status_inhibit = TRUE;
-	status_spot_remove_all ();
-
 	for (l = applets; l; l = l->next) {
 		AppletInfo *info = l->data;
 
 		panel_applet_save_position (info, info->gconf_key, TRUE);
-
-		if (info->type == APPLET_BONOBO) {
-			PanelAppletFrame *frame = info->data;
-
-			panel_applet_frame_set_clean_remove
-				(PANEL_APPLET_FRAME (frame), TRUE);
-		} else if (info->type == APPLET_SWALLOW) {
-			Swallow   *swallow = info->data;
-			GtkSocket *socket;
-
-			swallow->clean_remove = TRUE;
-
-			socket = GTK_SOCKET (swallow->socket);
-
-			if (socket->plug_window)
-                                XKillClient (GDK_DISPLAY (),
-					     GDK_WINDOW_XWINDOW(socket->plug_window));
-
-		}
 	}
 
+	panels_to_destroy = g_slist_copy (panel_list);
+
+	for (l = panels_to_destroy; l; l = l->next) {
+		PanelData *pd = l->data;
+
+		if (pd->panel)
+			gtk_widget_destroy (pd->panel);
+	}
+
+	g_slist_free (panels_to_destroy);
+
 	gnome_config_sync ();
 
-	xstuff_unsetup_desktop_area ();
-			
 	panel_shell_unregister ();
 	
-	gtk_main_quit();
+	gtk_main_quit ();
 
 	return TRUE;
 }
@@ -300,8 +265,12 @@ panel_session_die (GnomeClient *client,
 void
 panel_quit (void)
 {
-	gnome_client_request_save (client, GNOME_SAVE_BOTH, 1,
-				   GNOME_INTERACT_ANY, 0, 1);
+	/* Only request a Global save. We only want a Local
+	 * save if the user selects 'Save current setup'
+	 * from the dialog.
+	 */
+	gnome_client_request_save (
+		client, GNOME_SAVE_GLOBAL, 1, GNOME_INTERACT_ANY, 0, 1);
 }
 
 #ifdef FIXME
